@@ -74,13 +74,15 @@ static gchar *indicator_order[] = { "libayatana-application.so", "libayatana-mes
 
 static gchar *blacklist_applets[] = { "nm-applet", "chrome_status_icon_1", "chrome_status_icon_2",  0 };
 
-BudgiePanelPosition orient = BUDGIE_PANEL_POSITION_NONE;
-static guint current_icon_size;
-static guint panel_size;
-static guint icon_size;
-static guint small_icon_size;
+void calc_default_icon_size(void);
 
-GtkPackDirection packdirection = GTK_ORIENTATION_HORIZONTAL;
+BudgiePanelPosition orient = BUDGIE_PANEL_POSITION_NONE;
+static int current_icon_size;
+static int panel_size;
+static int icon_size;
+static int small_icon_size;
+
+GtkPackDirection packdirection = (GtkPackDirection)GTK_ORIENTATION_HORIZONTAL;
 
 #define MENU_DATA_INDICATOR_OBJECT "indicator-object"
 #define MENU_DATA_INDICATOR_ENTRY "indicator-entry"
@@ -97,8 +99,30 @@ void calc_default_icon_size() {
                 if (current_icon_size < 16) current_icon_size = 16;
         }
 }
+
+static void resize_image(GtkImage *image, __attribute__((unused)) gpointer user_data)
+{
+        GdkPixbuf *pixbuf;
+        GdkPixbuf *scaled_pixbuf;
+        g_debug("zzz resize_image");
+
+        pixbuf = gtk_image_get_pixbuf (image);
+
+        if (pixbuf == NULL) return;
+
+        if (gdk_pixbuf_get_height(pixbuf) != (current_icon_size-2)) {
+                scaled_pixbuf = gdk_pixbuf_scale_simple( pixbuf,
+                        (int)((double)(current_icon_size-2) / gdk_pixbuf_get_height(pixbuf) * gdk_pixbuf_get_width(pixbuf)),
+                        (current_icon_size-2), GDK_INTERP_HYPER
+                        );
+
+                gtk_image_set_from_pixbuf (image, scaled_pixbuf);
+        }
+
+}
+
 static gboolean
-entry_resized (AppIndicatorApplet *applet, int panel_size, int icon_size, int small_icon_size, gpointer data)
+entry_resized (__attribute__((unused)) AppIndicatorApplet *applet, __attribute__((unused)) int panel_size, __attribute__((unused)) int icon_size, __attribute__((unused)) int small_icon_size, gpointer data)
 {
 	IndicatorObject *io = (IndicatorObject *)data;
 
@@ -114,6 +138,10 @@ entry_resized (AppIndicatorApplet *applet, int panel_size, int icon_size, int sm
 		IndicatorObjectEntry * entrydata = (IndicatorObjectEntry *)entry->data;
 		if (entrydata->image != NULL) {
 			/* Resize to fit panel */
+
+                        if (gtk_image_get_storage_type(entrydata->image) == GTK_IMAGE_PIXBUF) {
+                                resize_image(entrydata->image, NULL);
+                        }
 			gtk_image_set_pixel_size (entrydata->image, current_icon_size);
 		}
 	}
@@ -241,6 +269,7 @@ static void sensitive_cb(GObject *obj, __attribute__((unused)) GParamSpec *pspec
         return;
 }
 
+
 static void entry_activated(GtkWidget *widget, gpointer user_data)
 {
         g_return_if_fail(GTK_IS_WIDGET(widget));
@@ -248,9 +277,9 @@ static void entry_activated(GtkWidget *widget, gpointer user_data)
         g_return_if_fail(INDICATOR_IS_OBJECT(pio));
         IndicatorObject *io = INDICATOR_OBJECT(pio);
 
-        return indicator_object_entry_activate(io,
-                                               (IndicatorObjectEntry *)user_data,
-                                               gtk_get_current_event_time());
+        indicator_object_entry_activate(io,
+                                        (IndicatorObjectEntry *)user_data,
+                                        gtk_get_current_event_time());
 }
 
 static gboolean entry_scrolled(GtkWidget *menuitem, GdkEventScroll *event,
@@ -336,6 +365,14 @@ static void entry_added(IndicatorObject *io, IndicatorObjectEntry *entry, GtkWid
 
         if (entry->image != NULL) {
                 g_debug("zzz have an image");
+                if (gtk_image_get_storage_type(entry->image) == GTK_IMAGE_PIXBUF) {
+                        g_debug("zzz have a pixbuf based image");
+                        g_signal_connect(G_OBJECT(entry->image),
+                                 "notify::pixbuf",
+                                 G_CALLBACK(resize_image),
+                                 NULL);
+                        resize_image(entry->image, NULL);
+                }
                 gtk_image_set_pixel_size(entry->image, current_icon_size);
                 gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(entry->image), FALSE, FALSE, 1);
                 if (gtk_widget_get_visible(GTK_WIDGET(entry->image))) {
@@ -488,6 +525,9 @@ static void entry_removed_cb(GtkWidget *widget, gpointer userdata)
                 g_signal_handlers_disconnect_by_func(G_OBJECT(entry->image),
                                                      G_CALLBACK(sensitive_cb),
                                                      widget);
+                g_signal_handlers_disconnect_by_func(G_OBJECT(entry->image),
+                                                     G_CALLBACK(resize_image),
+                                                     widget);
         }
 
         gtk_widget_destroy(widget);
@@ -728,7 +768,7 @@ static gboolean load_module(const gchar *name, AppIndicatorApplet *applet, GtkWi
         return TRUE;
 }
 
-void update_panel_size(AppIndicatorApplet *applet, int u_panel_size, int u_icon_size, int u_small_icon_size){
+void update_panel_size(__attribute__((unused)) AppIndicatorApplet *applet, int u_panel_size, int u_icon_size, int u_small_icon_size){
         g_debug("Panel size %d", u_panel_size);
         g_debug("icon size %d", u_icon_size);
         g_debug("small size %d", u_small_icon_size);
@@ -787,11 +827,8 @@ void hotkey_filter(char *keystring G_GNUC_UNUSED, gpointer data)
 gboolean menubar_on_draw(GtkWidget *widget, cairo_t *cr, GtkWidget *menubar)
 {
         if (gtk_widget_has_focus(menubar))
-                gtk_paint_focus(gtk_widget_get_style(widget),
+                gtk_render_focus(gtk_widget_get_style_context(widget),
                                 cr,
-                                gtk_widget_get_state(menubar),
-                                widget,
-                                "menubar-applet",
                                 0,
                                 0,
                                 -1,
